==> acsf-7.x-1.59.tar.gz: acsf/src/Acquia/Acsf/AcsfSjApiClient.php <==
<?php

namespace Acquia\Acsf;

/**
 * Provides a Scheduled Jobs API client.
 */
class AcsfSjApiClient implements AcsfSjClientInterface {

  /**
   * Scheduled Job's binaries path.
   *
   * @var string
   */
  private $binary;

  /**
   * Constructs the ACSF SJ Client.
   */
  public function __construct() {
    $this->binary = acsf_sj_get_sjadd_path();
  }

  /**
   * {@inheritdoc}
   */
  public function addJob($command, $reason, $timestamp, $domain, $timeout, $drush_executable, $drush_options) {
    try {
      $this->inputValidation($command, $timestamp, $domain, $timeout, $drush_executable, $drush_options);
    }
    catch (\Exception $e) {
      watchdog('acsf_sj', 'Unable to add the scheduled job: @message', array('@message' => $e->getMessage()), WATCHDOG_ERROR);
      return FALSE;
    }

    $sjadd_arguments = $this->prepareCommandArguments($command, $reason, $timestamp, $domain, $timeout, $drush_executable, $drush_options);

    return $this->exec($sjadd_arguments);
  }

  /**
   * Executes the sjadd command.
   *
   * @param string $command_arguments
   *   The sjadd command arguments and options.
   *
   * @return bool
   *   Whether the command execution was successful.
   *
   * @throws \RuntimeException
   *   If the sjadd binary is missing.
   */
  protected function exec($command_arguments) {
    if ($this->binary == NULL) {
      $message = 'sjadd binary is missing, preventing adding a scheduled job';
      throw new \RuntimeException($message);
    }
    if (!is_executable($this->binary)) {
      $message = 'sjadd binary is missing, preventing adding a scheduled job';
      throw new \RuntimeException($message);
    }

    $command = "$this->binary $command_arguments";

    $success = FALSE;
    // Possibly one retry.
    for ($retry = 0; $retry < 2 && !$success; $retry++) {
      exec(
        "timeout 10 $command",
        $output,
        $exit_code
      );
      $success = $exit_code === 0;
      watchdog('acsf_sj', 'SJ command "@command" has completed with exit code @exit_code.', array(
        '@command' => $command,
        '@exit_code' => $exit_code,
      ), $success ? WATCHDOG_NOTICE : WATCHDOG_ERROR);
      if (!$success) {
        // Retry after half of a second.
        usleep(500000);
      }
    }
    return $success;
  }

  /**
   * Validates/completes the sjadd input.
   *
   * @param string $command
   *   A drush command to run.
   * @param int $timestamp
   *   Unix timestamp when the command should be run or NULL to run ASAP.
   * @param string $domain
   *   The domain to use when calling the drush command or NULL for the class
   *   to determine.
   * @param int $timeout
   *   How long in seconds the process should be allowed to run or NULL for
   *   system default.
   * @param string $drush_executable
   *   The drush binary to use, 'drush' by default. i.e. drush9.
   * @param string $drush_options
   *   A list of drush options that will be applied to the drush command. If
   *   none are provided, "-y" will be used.
   *
   * @throws \InvalidArgumentException
   *   If the input arguments are not all valid.
   */
  protected function inputValidation($command, &$timestamp, &$domain, $timeout, &$drush_executable, &$drush_options) {
    global $base_url;
    $domain = (!empty($domain)) ? $domain : $base_url;

    if (is_null($timestamp)) {
      $timestamp = time();
    }
    $drush_executable = (!empty($drush_executable)) ? $drush_executable : 'drush';

    if (is_null($drush_options)) {
      $drush_options = '-y';
    }

    if (empty($command) || !is_string($command)) {
      throw new \InvalidArgumentException('The command argument must be a non-empty string.');
    }
    if ((!is_null($timestamp) && !is_numeric($timestamp)) || intval($timestamp) < 0) {
      throw new \InvalidArgumentException('The timestamp argument must be a positive integer.');
    }

    if (!is_null($domain) && (empty($domain) || !is_string($domain))) {
      throw new \InvalidArgumentException('The domain argument must be a non-empty string.');
    }

    if (!is_null($timeout) && (!is_numeric($timeout) || intval($timeout) < 0)) {
      throw new \InvalidArgumentException('The timeout argument must be a positive integer.');
    }

    if (!is_null($drush_executable) && (empty($drush_executable) || !is_string($drush_executable))) {
      throw new \InvalidArgumentException('The drush_executable argument must be a non-empty string.');
    }

    if (empty($drush_options) || !is_string($drush_options)) {
      throw new \InvalidArgumentException('The drush_options argument must be a non-empty string.');
    }
  }

  /**
   * Compiles the sjadd command.
   *
   * @param string $command
   *   A drush command to run.
   * @param string $reason
   *   The purpose of this job.
   * @param int $timestamp
   *   Unix timestamp when the command should be run or NULL to run ASAP.
   * @param string $domain
   *   The domain to use when calling the drush command or NULL for the class
   *   to determine.
   * @param int $timeout
   *   How long in seconds the process should be allowed to run or NULL for
   *   system default.
   * @param string $drush_executable
   *   The drush binary to use, 'drush' by default. i.e. drush9.
   * @param string $drush_options
   *   A list of drush options that will be applied to the drush command. If
   *   none are provided, "-y" will be used.
   *
   * @return string
   *   The sjadd command.
   */
  protected function prepareCommandArguments($command, $reason, $timestamp, $domain, $timeout, $drush_executable, $drush_options) {
    /*
     * Options for sjadd should be placed before its arguments so as to not be
     * mistaken for drush options.
     *
     * Argument order:
     * TIMESTAMP DOMAIN [DRUSH_COMMAND] [DRUSH_EXECUTABLE] [DRUSH_OPTIONS...]
     */
    $command_arguments = '';
    if (!empty($reason)) {
      $command_arguments .= sprintf("--reason='%s' ", $reason);
    }
    if (!empty($timeout) && is_numeric($timeout)) {
      $command_arguments .= sprintf('--max-exec-time=%d ', $timeout);
    }
    $command_arguments .= sprintf("%d %s '%s' '%s' '%s'", intval($timestamp), $domain, $command, $drush_executable, $drush_options);

    return $command_arguments;
  }

}

==> acsf-7.x-1.60.tar.gz: acsf/src/Acquia/Acsf/AcsfSjApiClient.php <==
<?php

namespace Acquia\Acsf;

/**
 * Provides a Scheduled Jobs API client.
 */
class AcsfSjApiClient implements AcsfSjClientInterface {

  /**
   * Scheduled Job's binaries path.
   *
   * @var string
   */
  private $binary;

  /**
   * Constructs the ACSF SJ Client.
   */
  public function __construct() {
    $this->binary = acsf_sj_get_sjadd_path();
  }

  /**
   * {@inheritdoc}
   */
  public function addJob($command, $reason, $timestamp, $domain, $timeout, $drush_executable, $drush_options) {
    try {
      $this->inputValidation($command, $timestamp, $domain, $timeout, $drush_executable, $drush_options);
    }
    catch (\Exception $e) {
      watchdog('acsf_sj', 'Unable to add the scheduled job: @message', array('@message' => $e->getMessage()), WATCHDOG_ERROR);
      return FALSE;
    }

    $sjadd_arguments = $this->prepareCommandArguments($command, $reason, $timestamp, $domain, $timeout, $drush_executable, $drush_options);

    return $this->exec($sjadd_arguments);
  }

  /**
   * Executes the sjadd command.
   *
   * @param string $command_arguments
   *   The arguments to the sjadd command, already escaped for shell execution.
   *
   * @return bool
   *   Whether the command was executed successfully.
   */
  protected function exec($command_arguments) {
    $error = '';
    if (!$this->binary) {
      $error = "The installation path for the ACSF 'sjadd' binary cannot be determined.";
    }
    elseif (!is_executable($this->binary)) {
      $error = "The ACSF 'sjadd' binary is missing or not executable, preventing adding a scheduled job.";
    }

    $exit_code = -1;
    if (!$error) {
      $command = sprintf('%s %s', $this->binary, $command_arguments);
      // Possibly one retry.
      for ($retry = 0; $retry < 2 && $exit_code !== 0; $retry++) {
        if ($retry) {
          watchdog('acsf_sj', "Command '@command' will be re-run; it exited with code @code:\n@error", array(
            '@command' => $command,
            '@code' => $exit_code,
            '@error' => $error,
          ), WATCHDOG_WARNING);
          // Retry after half of a second.
          usleep(500000);
        }

        // We're hardcoding a timeout of 10 seconds for just scheduling the job
        // (rather than for executing the scheduled job). Redirect stdout to
        // /dev/null als stderr to stdout, because odd errors from the sjadd
        // binary are expected on stderr. More logging is done by sjadd itself.
        exec(
          "AH_SITE_GROUP={$_ENV['AH_SITE_GROUP']} AH_SITE_ENVIRONMENT={$_ENV['AH_SITE_ENVIRONMENT']} timeout 10 $command 2>&1 1>/dev/null",
          $output,
          $exit_code
        );
        if ($exit_code !== 0) {
          $error = implode("\n", $output);
        }
      }
    }

    if ($exit_code !== 0) {
      // This message is not completely accurate for code -1, but that's OK.
      watchdog('acsf_sj', "Command '@command' exited with code @code:\n@error", array(
        '@command' => $command ?? '-',
        '@code' => $exit_code,
        '@error' => $error,
      ), WATCHDOG_ERROR);
    }
    return $exit_code === 0;
  }

  /**
   * Validates/completes the sjadd input.
   *
   * @param string $command
   *   A drush command to run.
   * @param int $timestamp
   *   Unix timestamp when the command should be run or NULL to run ASAP.
   * @param string $domain
   *   The domain to use when calling the drush command or NULL for the class
   *   to determine.
   * @param int $timeout
   *   How long in seconds the process should be allowed to run or NULL for
   *   system default.
   * @param string $drush_executable
   *   The drush binary to use, 'drush' by default. i.e. drush9.
   * @param string $drush_options
   *   A list of drush options that will be applied to the drush command. If
   *   none are provided, "-y" will be used.
   *
   * @throws \InvalidArgumentException
   *   If the input arguments are not all valid.
   */
  protected function inputValidation($command, &$timestamp, &$domain, $timeout, &$drush_executable, &$drush_options) {
    if (is_null($timestamp)) {
      $timestamp = time();
    }
    $drush_executable = (!empty($drush_executable)) ? $drush_executable : 'drush';
    // We can use $_SERVER['HTTP_HOST'] because (as long as) our sites.php
    // makes sure it's set.
    $domain = $domain ?: $_SERVER['HTTP_HOST'];
    if (is_null($drush_options)) {
      $drush_options = '-y';
    }

    if (empty($command) || !is_string($command)) {
      throw new \InvalidArgumentException('The command argument must be a non-empty string.');
    }
    if ((!is_null($timestamp) && !is_numeric($timestamp)) || intval($timestamp) < 0) {
      throw new \InvalidArgumentException('The timestamp argument must be a positive integer.');
    }

    if (!is_null($domain) && (empty($domain) || !is_string($domain))) {
      throw new \InvalidArgumentException('The domain argument must be a non-empty string.');
    }

    if (!is_null($timeout) && (!is_numeric($timeout) || intval($timeout) < 0)) {
      throw new \InvalidArgumentException('The timeout argument must be a positive integer.');
    }

    if (!is_null($drush_executable) && (empty($drush_executable) || !is_string($drush_executable))) {
      throw new \InvalidArgumentException('The drush_executable argument must be a non-empty string.');
    }

    if (empty($drush_options) || !is_string($drush_options)) {
      throw new \InvalidArgumentException('The drush_options argument must be a non-empty string.');
    }
  }

  /**
   * Compiles the sjadd command.
   *
   * @param string $drush_command
   *   A drush command to run.
   * @param string $reason
   *   The purpose of this job.
   * @param int $timestamp
   *   Unix timestamp when the command should be run or NULL to run ASAP.
   * @param string $domain
   *   The domain to use when calling the drush command or NULL for the class
   *   to determine.
   * @param int $timeout
   *   How long in seconds the process should be allowed to run or NULL for
   *   system default.
   * @param string $drush_executable
   *   The drush binary to use, 'drush' by default. i.e. drush9.
   * @param string $drush_options
   *   A list of drush options that will be applied to the drush command. If
   *   none are provided, "-y" will be used.
   *
   * @return string
   *   The sjadd command.
   */
  protected function prepareCommandArguments($drush_command, $reason, $timestamp, $domain, $timeout, $drush_executable, $drush_options) {
    /*
     * Options for sjadd should be placed before its arguments so as to not be
     * mistaken for drush options.
     *
     * Argument order:
     * TIMESTAMP DOMAIN [DRUSH_COMMAND] [DRUSH_EXECUTABLE] [DRUSH_OPTIONS...]
     */
    $command_arguments = '';
    if (!empty($reason)) {
      $command_arguments .= sprintf('--reason=%s ', escapeshellarg($reason));
    }
    if (!empty($timeout) && is_numeric($timeout)) {
      $command_arguments .= sprintf('--max-exec-time=%ds ', $timeout);
    }
    $command_arguments .= sprintf(
      '%d %s %s %s %s',
      intval($timestamp),
      escapeshellarg($domain),
      escapeshellarg($drush_command),
      escapeshellarg($drush_executable),
      escapeshellarg($drush_options)
    );

    return $command_arguments;
  }

}
